//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: ClientSideFrame.java,v 1.1.1.1 2002/06/10 20:53:31 jgiacomo Exp $
//
/*
 * ClientSideFrame.java
 *
 * Created on January 2, 2002, 4:07 PM
 */

package siena.xml.demo;

import siena.Notification;

/**
 *
 * @author  Chris Corliss
 * @version 0.1
 */

public class ClientSideFrame extends javax.swing.JFrame implements siena.Notifiable {

    siena.xml.XMLClient xmlInterface = null;
    
    String subscription = null;
    
    /** Creates new form ClientSideFrame */
    public ClientSideFrame() {
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jToolBar1 = new javax.swing.JToolBar();
        serverButton = new javax.swing.JButton();
        xschemaButton = new javax.swing.JButton();
        subscribeButton = new javax.swing.JButton();
        unsubscribeButton = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JSeparator();
        exitButton = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        subscriptionView = new javax.swing.JTextArea();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        xpathLabel = new javax.swing.JTextField();
        
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        
        serverButton.setToolTipText("Configure the SIENA server");
        serverButton.setText("Server");
        serverButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                serverButtonActionPerformed(evt);
            }
        });
        
        jToolBar1.add(serverButton);
        
        xschemaButton.setToolTipText("Set where the XSchema file is");
        xschemaButton.setText("XSchema");
        xschemaButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xschemaButtonActionPerformed(evt);
            }
        });
        
        jToolBar1.add(xschemaButton);
        
        subscribeButton.setToolTipText("Subscribe to an event");
        subscribeButton.setText("Subscribe");
        subscribeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                subscribeButtonActionPerformed(evt);
            }
        });
        
        jToolBar1.add(subscribeButton);
        
        unsubscribeButton.setToolTipText("Unsubscribe from an event");
        unsubscribeButton.setText("unsubscribe");
        unsubscribeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unsubscribeButtonActionPerformed(evt);
            }
        });
        
        jToolBar1.add(unsubscribeButton);
        
        jToolBar1.add(jSeparator1);
        
        exitButton.setToolTipText("Quit the program");
        exitButton.setText("Exit");
        exitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitButtonActionPerformed(evt);
            }
        });
        
        jToolBar1.add(exitButton);
        
        getContentPane().add(jToolBar1, java.awt.BorderLayout.NORTH);
        
        subscriptionView.setEditable(false);
        jScrollPane1.setViewportView(subscriptionView);
        
        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);
        
        jLabel1.setText("Current Subscription");
        jPanel1.add(jLabel1);
        
        xpathLabel.setEditable(false);
        xpathLabel.setColumns(22);
        xpathLabel.setText("NOT SET");
        jPanel1.add(xpathLabel);
        
        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);
        
        pack();
    }//GEN-END:initComponents

    private void clearMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearMenuItemActionPerformed
        subscriptionView.setText("");
    }//GEN-LAST:event_clearMenuItemActionPerformed

    private void exitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitButtonActionPerformed
        this.exitForm(null);
    }//GEN-LAST:event_exitButtonActionPerformed

    private void unsubscribeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unsubscribeButtonActionPerformed
        if (xmlInterface == null) {
            javax.swing.JOptionPane.showMessageDialog(this,"You must first configure the Siena server!");
            return;
        }
        
        if (xpathLabel.getText().equals("NOT SET")) {
            subscriptionView.append("No subscription to unsubscribe to.\n");
            return;
        }
        
        try {
            xmlInterface.unsubscribe(xpathLabel.getText(), this);
        } catch (siena.SienaException exception) {
            subscriptionView.append("Error while trying to unsubscribe from "+xpathLabel.getText()+"\n");
            subscriptionView.append("Error is: "+exception.getMessage()+"\n");                    
        }
        
        xpathLabel.setText("NOT SET");
    }//GEN-LAST:event_unsubscribeButtonActionPerformed

    private void subscribeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_subscribeButtonActionPerformed
        if (xmlInterface == null) {
            javax.swing.JOptionPane.showMessageDialog(this,"You must first configure the Siena server!");
            return;
        }
        
        if (!xpathLabel.getText().equals("NOT SET")) {
            javax.swing.JOptionPane.showMessageDialog(this,"Please unsubscribe from current subscription!");
            return;
        }

        subscription = javax.swing.JOptionPane.showInputDialog(this, "Enter the XPath");
        if (subscription != null) {
            xpathLabel.setText(subscription);
        } else {
            xpathLabel.setText("NOT SET");
            return;
        }
        
        try {
            xmlInterface.subscribe(subscription, this);
        } catch (siena.SienaException exception) {
            subscriptionView.append("Unable to subscribe to "+subscription+"\n");
            subscriptionView.append("Error is: "+exception.getMessage()+"\n");
            xpathLabel.setText("NOT SET");
        }
    }//GEN-LAST:event_subscribeButtonActionPerformed

    private void xschemaButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xschemaButtonActionPerformed
        String filename = null;
        javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser();
        int returnValue = 0;

        if (xmlInterface == null) {
            javax.swing.JOptionPane.showMessageDialog(this,"You must first configure the Siena server!");
            return;
        }
        
        if (lastDir != null) {
            fileChooser.setCurrentDirectory(lastDir);
        }
        
        returnValue = fileChooser.showOpenDialog(this);
        if (returnValue == javax.swing.JFileChooser.APPROVE_OPTION) {
            lastDir = fileChooser.getSelectedFile();
            filename = lastDir.getAbsolutePath();
        } else {
            return;
        }
       
        try {
            xmlInterface.setStyleSheet(filename);
        } catch (java.io.IOException exception) {
            subscriptionView.append("Error while dealing with the XSchema file: "+exception.getMessage()+"\n");
            subscriptionView.append("XSchema file not set.\n");
            return;
        }
    }//GEN-LAST:event_xschemaButtonActionPerformed

    private void serverButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_serverButtonActionPerformed
        String server = null;
        int port = 0;
        
        SienaConfigurationDialog dialog = new SienaConfigurationDialog(this, true);
        dialog.setLocationRelativeTo(this);
        dialog.setVisible(true);
        server = dialog.getSienaServer();
        port = dialog.getPort();
        if (server == null || port == 0) {
            subscriptionView.append("Siena server configuration canceled\n");
            return;
        }
        
        try {
            subscriptionView.append("Trying to connect to Siena Server: "+server+" at "+port+"\n");
            xmlInterface = new siena.xml.XMLClient("tcp:"+server+":"+port);
        } catch (siena.comm.InvalidSenderException exception) {
            subscriptionView.append("Invalid siena server. "+exception.toString()+"\n");
            return;
        }
        
        subscriptionView.append("Siena server identified.\n");
    }//GEN-LAST:event_serverButtonActionPerformed

    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        new ClientSideFrame().show();
    }

    /** sends a <code>Notification</code> to this <code>Notifable</code>
     *
     * Since version 1.0.1 of the Siena API it is safe to modify the
     * Notification object received through this method.  Note that:
     * <ol>
     *
     * <li><em>any</em> previous version of the Siena API assumes that
     *     clients <em>do not modify</em> these notifications;
     *
     * <li>the current solution incurrs in an unnecessary cost by
     *     having to duplicate every notification.  Therefore, it
     *     is a <em>temporary solution</em>.  The plan is to
     *     implement <em>immutable</em> notifications and to pass
     *     those to subscribers.
     *
     * </ol>
     * necessary duplication of notifications can be expensive,
     * especially if the same notification must be copied to numerous
     * subscribers.
     *
     * @param n notification passed to the notifiable
     * @see Siena#subscribe(Filter,Notifiable)
     */
    public void notify(Notification n) throws siena.SienaException {
        subscriptionView.append(n.toString()+"\n");
    }    

    /** sends a sequence of <code>Notification</code>s to this
     * <code>Notifable</code>
     *
     * Since version 1.0.1 of the Siena API it is safe to modify the
     * Notification objects received through this method.  Please
     * read the notes in the above documentation of {@link
     * #notify(Notification)}, which apply to this method as well.
     *
     * @param s sequence of notifications passed to the notifiable
     * @see Siena#subscribe(Pattern,Notifiable)
     */
    public void notify(Notification[] s) throws siena.SienaException {
        for (int i = 0; i < s.length; i++) {
            subscriptionView.append(s[i].toString()+"\n");
        }            
    }
    
    private java.io.File lastDir;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JButton serverButton;
    private javax.swing.JButton xschemaButton;
    private javax.swing.JButton subscribeButton;
    private javax.swing.JButton unsubscribeButton;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JButton exitButton;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea subscriptionView;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JTextField xpathLabel;
    // End of variables declaration//GEN-END:variables

}
