//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: InequalityCondition.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/*
 * InequalityCondition.java
 *
 * Created on November 16, 2001, 11:11 PM
 */

package siena.xml.rules;

/** This class checks for inequality conditions, such as less than.
 *
 * @author Chris Corliss
 * @version 0.1
 */
public class InequalityCondition implements Condition {
    
    /** Flag to indicate if less than comparison is to be used.
     */    
    private boolean isLessThan = false;
    /** Flag that indicates if double values should be used in the comparison.
     */    
    private boolean useDoubleValue = false;
    /** THe integer value to compare against.
     */    
    private int value = 0;
    /** The double value to compare against.
     */    
    private double doubleValue = 0;
    
    /** Creates new InequalityCondition
     * @param value Value to compare against
     * @param testForLess Flag to indicate if the comparison is for less than the specified value.
     */
    public InequalityCondition(int value, boolean testForLess) {
        isLessThan = testForLess;
        this.value = value;
    }
    
    /** Constructor
     * @param value Value to compare against
     * @param testForLess Flag to indicate if the comparison is for less than the specified value.
     */    
    public InequalityCondition(double value, boolean testForLess) {
        isLessThan = testForLess;
        this.doubleValue = value;
        useDoubleValue = true;
    }
    
    /** Method to call to see if the object meets the condition specified by this object.
     * @param whatToEvaluate Object to evaluate the conditions against.
     * @return True if the object meets the conditions in this object.
     */    
    public boolean meetCondition(Object whatToEvaluate) {
        String stringValue = null;
        int valueToEval = 0;
        double doubleValueToEval = 0;
        
        if (whatToEvaluate == null) {
            return false;
        }
        
        if (whatToEvaluate instanceof org.w3c.dom.Node) {
            stringValue = siena.xml.DOMTreeProcessing.getNodeValue((org.w3c.dom.Node)whatToEvaluate);
        } else {
            stringValue = whatToEvaluate.toString();
        }
        
        if (useDoubleValue) {
            try {
                doubleValueToEval = Double.parseDouble(stringValue);
                if (doubleValue > doubleValueToEval && isLessThan) {
                    return true;
                } else if(doubleValue < doubleValueToEval && !isLessThan) {
                    return true;
                }
            } catch (Exception exception) {
                // If there is an exception then the wrong type
                // was passed in. By simply doing nothing the exception is taken care of
                // because the method will continue after the catch block and return false
                // to the caller which says that the condition failed. The
                // correct type to pass in is a string containing a number.
            }
        } else {
            try {
                valueToEval = Integer.parseInt(stringValue);
                if (value > valueToEval && isLessThan) {
                    return true;
                } else if(value < valueToEval && !isLessThan) {
                    return true;
                }
            } catch (Exception exception) {
                // If there is an exception then the wrong type
                // was passed in. By simply doing nothing the exception is taken care of
                // because the method will continue after the catch block and return false
                // to the caller which says that the condition failed. The
                // correct type to pass in is a string containing a number.
            }
        }
        return false;
    }
}
