//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: EqualityPredicateTest.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//

/*
 * EqualityPredicateTest.java
 * JUnit based test
 *
 * Created on December 2, 2001, 9:30 PM
 */                

package siena.xml.tests;

import siena.xml.xpath.EqualityPredicate;
import junit.framework.*;

/**
 *
 * @author unknown
 */                                
public class EqualityPredicateTest extends TestCase {
    
    public EqualityPredicateTest(java.lang.String testName) {
        super(testName);
    }        
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(EqualityPredicateTest.class);        
        return suite;
    }
    
    /** Test of getStringValue method, of class siena.xml.xpath.EqualityPredicate. */
    public void testGetStringValue() {
        EqualityPredicate predicate = null;
        
        System.out.println(" - Test: EqualityPredicate - getStringValue method.");
        
        predicate = new EqualityPredicate("name","bob");
        assertEquals("String value is incorrect.", "bob", predicate.getStringValue());
        
    }
    
    /** Test of getIntValue method, of class siena.xml.xpath.EqualityPredicate. */
    public void testGetIntValue() {
        EqualityPredicate predicate = null;

        System.out.println(" - Test: EqualityPredicate - getIntValue method.");
        
        predicate = new EqualityPredicate("name", "1");
        assertEquals("Integer value is incorrect.", 1, predicate.getIntValue());
    }
    
    /** Test of getDoubleValue method, of class siena.xml.xpath.EqualityPredicate. */
    public void testGetDoubleValue() {
        EqualityPredicate predicate = null;
        System.out.println(" - Test: EqualityPredicate - getDoubleValue method.");

        predicate = new EqualityPredicate("name", "0.01");
        assertEquals("Double value is incorrect.", 0.01, predicate.getDoubleValue(),(0.01-predicate.getDoubleValue()));
    }
    
    /** Test of isIntegerComparison method, of class siena.xml.xpath.EqualityPredicate. */
    public void testIsIntegerComparison() {
        EqualityPredicate predicate = null;

        System.out.println(" - Test: EqualityPredicate - isIntegerComparison method.");
        
        predicate = new EqualityPredicate("name","bob");
        assertTrue("A string comparison was thought to be an integer comparison.", !predicate.isIntegerComparison());
        predicate = new EqualityPredicate("name","1");
        assertTrue("An integer comparison was not found when there was one.", predicate.isIntegerComparison());
    }
    
    /** Test of isDoubleComparison method, of class siena.xml.xpath.EqualityPredicate. */
    public void testIsDoubleComparison() {
        EqualityPredicate predicate = null;

        System.out.println(" - Test: EqualityPredicate - isDoubleComparison method.");
        
        predicate = new EqualityPredicate("name","bob");
        assertTrue("A string comparison was thought to be a double comparison.", !predicate.isDoubleComparison());
        predicate = new EqualityPredicate("name",".01");
        assertTrue("A double comparison was not found when there was one.", predicate.isDoubleComparison());
    }
    
    /** Test of isStringComparison method, of class siena.xml.xpath.EqualityPredicate. */
    public void testIsStringComparison() {
        EqualityPredicate predicate = null;

        System.out.println(" - Test: EqualityPredicate - isStringComparison method.");
        
        predicate = new EqualityPredicate("name","1");
        assertTrue("An integer comparison was thought to be a string comparison.", !predicate.isStringComparison());
        predicate = new EqualityPredicate("name","bob");
        assertTrue("A string comparison was not found when there was one.", predicate.isStringComparison());
    }
    
    /** Test of getTarget method, of class siena.xml.xpath.EqualityPredicate. */
    public void testGetTarget() {
        EqualityPredicate predicate = null;
        
        System.out.println(" - Test: EqualityPredicate - getTarget method.");
        
        predicate = new EqualityPredicate("test","passed");
        assertEquals("Target was not correctly set in the predicate.", "test", predicate.getTarget());
    }
}
