//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: TestNotifiableObject.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
package siena.xml.tests;

/**
 * This class is a generic class that is used to fulfill the
 * requirement of the XMLSubscriptionHandler constructor's
 * parameter for a Notifiable object. When one of it's notify
 * methods are called a flag is set to indicate that the object
 * was notified.
 * @author Chris Corliss
 * @version 0.1
 * @since 12-21-01
 * @see siena.Notifiable
 */
public class TestNotifiableObject implements siena.Notifiable {
    boolean wasNotified = false;

    /** This method is required by the siena.Notifiable interface.
     * The method sets the notified flag to true when called.
     * @param notification A Siena.Notification
     * @exception siena.SienaException This exception is not used in this method.
     */    
    public void notify(siena.Notification notification) throws siena.SienaException {
        wasNotified = true;
    }
    /** 
     * This method is required by the siena.Notifiable interface.
     * The method sets the notified flag to true when called.
     * @param s An array of Siena Notifications
     * @exception siena.SienaException This exception is not used in this method
     */    
    public void notify(siena.Notification[] s) throws siena.SienaException {
        wasNotified = true;
    }

    /** 
     * This method returns the state of the notified flag.
     * @return True or false depending of the object has been notified or not
     */    
    public boolean getNotified() {
        return wasNotified;
    }
    
    /**
     * This method sets the notified flag to false, in effect resetting the 
     * object.
     */
    public void resetNotifiedFlag() {
        wasNotified = false;
    }
}
