//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: XMLSubscriptionHandlerTest.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//

/*
 * XMLSubscriptionHandlerTest.java
 * JUnit based test
 *
 * Created on December 2, 2001, 9:26 PM
 */                

package siena.xml.tests;

import junit.framework.*;
import siena.AttributeValue;
import siena.AttributeConstraint;
import siena.Op;
import siena.Filter;
import siena.SienaException;
import siena.Notification;
import java.util.zip.GZIPInputStream;
import java.util.Vector;
import java.util.Enumeration;
import java.util.HashMap;
import siena.Notifiable;
import siena.xml.xpath.*;
import siena.xml.rules.Rule;

/**
 *
 * @author unknown
 */                                
public class XMLSubscriptionHandlerTest extends TestCase implements siena.xml.Observer {
    
    private siena.xml.XMLSubscriptionHandler handler = null;
    private boolean updateIsASuccess = false; 
    private TestNotifiableObject testObj = null;
    private siena.Notification event = null;
    private static final String XMLNOTIFICATION =
        "<testNotification><data>NOTIFICATION</data></testNotification>";

    public XMLSubscriptionHandlerTest(java.lang.String testName) {
        super(testName);
    }        
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(XMLSubscriptionHandlerTest.class); 
        return suite;
    }
    
    protected void setUp() throws Exception {
        super.setUp();

        java.io.StringWriter writer = null;
        java.io.PrintWriter printWriter = null;
        java.io.ObjectOutputStream output = null;
        java.io.ByteArrayOutputStream data = null;

        testObj = new TestNotifiableObject();
        handler = new siena.xml.XMLSubscriptionHandler(testObj);
        event = new siena.Notification();
        data = new java.io.ByteArrayOutputStream();
        try {
           output = new java.io.ObjectOutputStream(new java.util.zip.GZIPOutputStream(data)); 
           output.writeUTF(XMLNOTIFICATION);
           output.close();
        } catch (Exception exception) {
            writer = new java.io.StringWriter();
            printWriter = new java.io.PrintWriter(writer);
            writer.write("Exception caught during test run: ");
            writer.write("\n\n*******BEGIN EXCEPTION********\n\n");
            exception.printStackTrace(printWriter);
            writer.write("\n*******END EXCEPTION********\n");
            fail(writer.toString());
        }
        event.putAttribute("_xml", data.toByteArray());
    }

    public void update(siena.xml.XMLSubscriptionHandler handler) {
        updateIsASuccess = true;
    }

    /** Test of notify method, of class siena.xml.XMLSubscriptionHandler. */
    public void testNotify() {
        java.io.StringWriter writer = null;
        java.io.PrintWriter printWriter = null;

        System.out.println(" - Test: XMLSubscriptionHandler - notify method");
        
        System.out.println("Calling the test to set the style sheet...");
        testSetStylesheet();

        handler.attach(this);

        try {
            handler.buildFilter("testNotification[data=\"NOTIFICATION\"]");
            handler.notify(event);
        } catch (siena.SienaException exception) {
            writer = new java.io.StringWriter();
            printWriter = new java.io.PrintWriter(writer);
            writer.write("Exception caught during test run: ");
            writer.write("\n\n*******BEGIN EXCEPTION********\n\n");
            exception.printStackTrace(printWriter);
            writer.write("\n*******END EXCEPTION********\n");
            fail(writer.toString());
        }
        Assert.assertTrue("Test object was not notified.", updateIsASuccess);
        handler.detach(this);
    }
    
    /** Test of attach method, of class siena.xml.XMLSubscriptionHandler. */
    public void testAttach() {
        System.out.println(" - Test: XMLSubscriptionHandler - attach method");
        
        handler.attach(this);

        java.util.Enumeration observers = handler.observers();
        while (observers.hasMoreElements()) {
            Object testObj = observers.nextElement();
            if (testObj == this) {
                handler.detach(this);
                return;
            }
        }

        fail("The test case failed: No observer was added.");

    }
    
    /** 
     * Test of detach method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testDetach() {
        System.out.println(" - Test: XMLSubscriptionHandler - detach method");
        
        System.out.println("Calling the test for the attach method to make sure it is working....");
        this.testAttach();
        System.out.println("Calling the test for the observers method to make sure the test object is an observer...");
        this.testObservers();

        handler.detach(this);
        java.util.Enumeration observers = handler.observers();
        while (observers.hasMoreElements()) {
            Object testObj = observers.nextElement();
            if (testObj == this) {
                fail("The test case failed: Test observer was not removed.");
            }
        }
    }
    
    /** 
     * Test of inform method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testInform() {

        System.out.println(" - Test: XMLSubscriptionHandler - inform method");
        
        handler.attach(this);
        handler.inform();
        Assert.assertTrue("Test Observer was not informed.", updateIsASuccess);
        handler.detach(this);

    }
    
    /** 
     * Test of observers method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testObservers() {

        System.out.println(" - Test: XMLSubscriptionHandler - observer method");

        handler.attach(this);
        java.util.Enumeration observers = handler.observers();
        Assert.assertTrue("No observers after an attach operation has been performed.",observers.hasMoreElements());
        handler.detach(this);
        observers = handler.observers();
        Assert.assertTrue("Observers after an detach operation has been performed.",!(observers.hasMoreElements()));

    }
    
    /** 
     * Test of getNotification method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testGetNotification() {
        System.out.println(" - Test: XMLSubscriptionHandler - getNotification method");
        
        System.out.println("Calling the test for the notify method...");
        this.testNotify();
        // Add your test code below by replacing the default call to fail.
        Assert.assertNotNull("Notification was not set properly.", handler.getNotification());
    }
    
    /** 
     * Test of getNotifiable method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testGetNotifiable() {
        Object notifiable = handler.getNotifiable();

        System.out.println(" - Test: XMLSubscriptionHandler - getNotifiable method");
        
        Assert.assertEquals("The notifable returned is not the same one that was set.", testObj, notifiable);
        
    }
    
    /** 
     * Test of buildFilter method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testBuildFilter() {

        String xmlNotification = null; 
        siena.Filter[] filters = null;
        siena.Filter[] tempFilter = null;
        String xpath = null;
        java.util.Iterator iterator = null;
        java.io.StringWriter writer = null;
        java.io.PrintWriter printWriter = null;

        System.out.println(" - Test: XMLSubscriptionHandler - buildFilter method");

        filters = new siena.Filter[9];

        try {
            handler.setStylesheet(this.getClass().getResource("test.xsd").getFile());
        } catch (Exception exception) {
            fail("Unable to set the stylesheet for this test: "+
                    exception.getMessage());
        }

        try {
            xpath = "/testNotification[data = \"NOTIFICATION\"]";
            tempFilter = handler.buildFilter(xpath);
            filters[0] = tempFilter[0];
            xpath = "/testNotification/list/item[last()]";
            tempFilter = handler.buildFilter(xpath);
            filters[1] = tempFilter[0];
            xpath = "/testNotification/list/item[position() = 1]";
            tempFilter = handler.buildFilter(xpath);
            filters[2] = tempFilter[0];
            xpath = "/testNotification/list/item [1]";
            tempFilter = handler.buildFilter(xpath);
            filters[3] = tempFilter[0];
            xpath = "/testNotification/list[substring(item,\"test\")]";
            tempFilter = handler.buildFilter(xpath);
            filters[4] = tempFilter[0];
            xpath = "/testNotification/list/item[substring(name,\"test\") and value=\"1.00\"]";
            tempFilter = handler.buildFilter(xpath);
            filters[5] = tempFilter[0];
            xpath = "/testNotification/list/item[string-length(name) > \"1\"]";
            tempFilter = handler.buildFilter(xpath);
            filters[6] = tempFilter[0];
            xpath = "/testNotification/list[concat(item,\"test\",\"here\")]";
            tempFilter = handler.buildFilter(xpath);
            filters[7] = tempFilter[0];
            xpath = "/testNotification/list/item[value > 10.00]";
            tempFilter = handler.buildFilter(xpath);
            filters[8] = tempFilter[0];
        } catch (siena.SienaException exception) {
            writer = new java.io.StringWriter();
            printWriter = new java.io.PrintWriter(writer);
            writer.write("Exception caught during test run: ");
            writer.write("\n\n*******BEGIN EXCEPTION********\n\n");
            exception.printStackTrace(printWriter);
            writer.write("\n*******END EXCEPTION********\n");
            fail(writer.toString());
        }

        for (int i = 0; i < filters.length; i++) {
            iterator = filters[i].constraintNamesIterator();
            Assert.assertTrue("No constraints added for xpath: "+xpath, iterator.hasNext());
        }

        iterator = filters[0].constraintsIterator("data");
        Assert.assertTrue("No data constraint for xpath: "+xpath, iterator.hasNext());
    }

    /** 
     * Test of setStylesheet method, of class siena.xml.XMLSubscriptionHandler. 
     */
    public void testSetStylesheet() {
        java.io.StringWriter writer = null;
        java.io.PrintWriter printWriter = null;

        System.out.println(" - Test: XMLSubscriptionHandler - setStyleSheet method");
        
        try {
            handler.setStylesheet(this.getClass().getResource("test.xsd").getFile());
        } catch (Exception exception) {
            writer = new java.io.StringWriter();
            printWriter = new java.io.PrintWriter(writer);
            writer.write("Exception caught during test run: ");
            writer.write("\n\n*******BEGIN EXCEPTION********\n\n");
            exception.printStackTrace(printWriter);
            writer.write("\n*******END EXCEPTION********\n");
            fail(writer.toString());
        }
    }
}
