//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: BinaryNode.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
package siena.xml.xpath;

/** This object represents a node in a Binary Tree. It has two children, left and right,
 * and space to store a single user object.
 */
public class BinaryNode {
    /** Left child of the node
     */    
    private BinaryNode leftChild;
    /** Right child of the node
     */    
    private BinaryNode rightChild;
    /** User data
     */    
    private Object data;
    /** True if this node is a leaf
     */    
    private boolean isLeaf = false;    
    
    /** Constructor
     */    
    public BinaryNode() {
        leftChild = null;
        rightChild = null;
        data = null;
    }
    
    /** Sets the left child of this node
     * @param node Node to be set as the left child.
     */    
    public void setLeftChild(BinaryNode node) {
        leftChild = node;
    }
    
    /** Sets the right child of the node
     * @param node Node to be set as the right child.
     */    
    public void setRightChild(BinaryNode node) {
        rightChild = node;
    }
    
    /** Sets the user data
     * @param newData User data
     */    
    public void setData(Object newData) {
        data = newData;
    }
    
    /** Returns the left child of the node
     * @return BinaryNode object that is the left child of this node.
     */    
    public BinaryNode getLeftChild() {
        return leftChild;
    }
    
    /** Returns the right child of the node
     * @return BinaryNode object that is the right child of this node.
     */    
    public BinaryNode getRightChild() {
        return rightChild;
    }
    
    /** Return the user data
     * @return Object that was set by the previous call to setData.
     */    
    public Object getData() {
        return data;
    }
    
    /** Sets that this node is a leaf node.
     */    
    public void setLeaf() {
        isLeaf = true;
    }
    
    /** Returns if this node is a leaf node in the tree.
     * @return Returns true if this node is a leaf.
     */    
    public boolean isLeaf() {
        return isLeaf;
    }
}


