//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: CompoundPredicate.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/*
 * CompoundPredicate.java
 *
 * Created on November 25, 2001, 3:16 PM
 */

package siena.xml.xpath;

import java.util.Vector;

/** Object that represents a compound predicate: two or more predicates connected by
 * boolean operations. A binary tree is used to represent this predicate. This predicate
 * uses the preidcate parser to generate the binary tree.
 *
 * @author Chris Corliss
 * @version 0.1
 */

public class CompoundPredicate extends Predicate {

    private static final char TOKENDIVIDER = (char)0x00;
    private BinaryNode root;
    private PredicateParser parser;
    
    /** Creates new CompoundPredicate */

    public CompoundPredicate(String predicates) {
        super(Predicate.COMPOUND);
        java.io.DataInputStream dis = new java.io.DataInputStream(new java.io.ByteArrayInputStream(predicates.getBytes()));
        PredicateLexer lexer = new PredicateLexer(dis);
        parser = new PredicateParser(lexer);
        try {
            root = parser.predicate();
        } catch (Exception exception) {
            // Need to do error handling here... =(
            return;
        }        
        
        buildPredicateTree(root);
    }

    private void buildPredicateTree(BinaryNode node) {
        BinaryNode temp = null;
        String data = null;
        temp = node.getLeftChild();
        if (temp != null) {
            buildPredicateTree(temp);
        }        
        temp = node.getRightChild();       
        if (temp != null) {
            buildPredicateTree(temp);
        }
        data = (String)node.getData();
        if (data.equals("OR") || data.equals("AND")) {
            return;
        } else {
            node.setData(Predicate.buildPredicate(data));
            node.setLeaf();
        }
    }
    
    public BinaryNode getTree() { 
        return root;
    }          
}
