//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: Predicate.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/* Generated by Together */

package siena.xml.xpath;

/** Abstract class that also acts as a factory for predicates.
 */
public class Predicate {

    public static final int FUNCTION = 0;
    public static final int EQUALITY = 1;
    public static final int POSITION = 2;
    public static final int COMPOUND = 3;
    public static final int COMPARE  = 4;

    private int type;

    protected Predicate(int type) {
        this.type = type;
    }
    
    public int getType() {
        return type;
    }
    
    /** Given a string this method will construct a new predicate object based on the contents of the string.
     * Note that the braces '[' nad ']' must also be included in the string.
     * @param data String containing the predicate from the XPath statement.
     * @return New Predicate object that represents the predicate passed in as a string.
     */    
    public static Predicate buildPredicate(String data) {
        String trimedData = null;
        StringBuffer buffer = null;
        String what = null;
        String value = null;
        int andIndex = -1;
        int orIndex = -1;
        int position = -1;
        int index = -1;
        int startIndexOfOp = -1;
        int indexOfParen = -1;

        andIndex = data.indexOf("and");
        orIndex = data.indexOf("or");

        if (andIndex != -1 || orIndex != -1) {
            return (new CompoundPredicate(data));
        }
        
        trimedData = data.replace('[',' ').trim();
        trimedData = trimedData.replace(']',' ').trim();

        /**
         * Handle the different position predicates first since one form of it can
         * be confused as an equality predicate.
         */
        
        try {
            position = Integer.parseInt(trimedData);
            return new PositionPredicate(position);
        } catch (Exception ignore) {}
        
        if (trimedData.startsWith("position(") || trimedData.startsWith("last(") || trimedData.indexOf("TO") != -1) {
            return new PositionPredicate(trimedData);
        }
        
        index = trimedData.indexOf('<');
        if (index == -1) {
            index = trimedData.indexOf('>');
            if (index != -1) {
                return new ComparisonPredicate(trimedData);
            }
        } else {
            return new ComparisonPredicate(trimedData);
        }
                
        index = trimedData.indexOf('=');
        if (index != -1) {
            startIndexOfOp = index;
            if (trimedData.charAt(index-1) == '!') {
                startIndexOfOp--;
            }
            
            /**
             * If one of the parts of the equality is a function then a compound
             * predicate needs to be built to handle the function and other part
             * separately. If this was not done then the function name will be 
             * interpeted as a literal and not as a function.
             */
            
            indexOfParen = trimedData.indexOf('(');
            if (indexOfParen != -1) {/*
                buffer = new StringBuffer();
                buffer.append("[");
                buffer.append(trimedData.substring(0,startIndexOfOp));
                buffer.append(" and ");
                buffer.append(trimedData.substring(index+1));
                buffer.append("]");
                return new CompoundPredicate(buffer.toString());*/
                trimedData = trimedData.substring(index+1);
            } else {
                what = trimedData.substring(0,startIndexOfOp);
                value = trimedData.substring(index+1);
                return new EqualityPredicate(what, value);
            }
        }

        return (new FunctionPredicate(trimedData));
    }
}
