//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: Step.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/* Generated by Together */

package siena.xml.xpath;

/** This object represents a single step in an XPath statement.
 */
public class Step {
    
    /** Tree axis that might have been specified at the beginning of the step.
     */    
    private String axis = null;
    /** Node test portion of the step
     */    
    private String nodeTest = null;
    /** The optional predicate in the step
     */    
    private Predicate predicate = null;
    /** Flag to indicate if the nodeTest is a function and not an XML tag.
     */    
    private boolean nodeTestIsAFunction = false;

    public Step(String stepInfo) {
        String step = stepInfo;
        int axisEnd = step.indexOf("::");
        if (axisEnd != -1) {
            axis = step.substring(0,axisEnd);
            step = step.substring(axisEnd+2);
        }
        int predicateStart = step.indexOf('[');
        if (predicateStart != -1) {
            predicate = Predicate.buildPredicate(step.substring(predicateStart));
            nodeTest = step.substring(0,predicateStart).trim();
        } else {
            nodeTest = step.trim();
        }
        
        if (nodeTest.indexOf('(')  != -1) {
            nodeTestIsAFunction = true;
        }
    }
    
    public boolean hasPredicate() {
        if (predicate != null) {
            return true;
        }
        return false;
    }
    
    public String getAxis() {
        return axis;
    }
    
    public String getNodeTest() {
        return nodeTest;
    }
           
    public Predicate getPredicate() {
        return predicate;
    }
    
    public boolean isNodeTestAFunction() {
        return nodeTestIsAFunction;
    }
}
